import asyncio
from datetime import datetime, timedelta
from telegram import Update
from telegram.ext import (
    Application,
    CommandHandler,
    MessageHandler,
    filters,
    ContextTypes
)
import aiohttp
import nest_asyncio
from functools import wraps

# BOT_TOKEN = "7664853316:AAEP2dAb3GuWLib6eJB1vu8DIR65HqlIBRA"
# KEYWORD_LIST_URL = "https://loopaddon.uk/files/PJ/BOT_DATA/loop_bot_keywords.json"

# BOT_TOKEN = "YOUR_BOT_TOKEN_HERE"  # Replace with your actual token
# Loop Gatekeeper bot
BOT_TOKEN = "7664853316:AAEP2dAb3GuWLib6eJB1vu8DIR65HqlIBRA"
# Loop Development
# BOT_TOKEN = "7309367551:AAGJNNPPQroL7yqw7U7eeUmsOXrKVQgzAw0" 
KEYWORD_LIST_URL = "https://loopaddon.uk/files/PJ/BOT_DATA/loop_bot_keywords.json"

new_users = {}
user_warnings = {}

check_list1 = []
check_list2 = []
check_list3 = []
allowed_users = []

def admin_only(func):
    @wraps(func)
    async def wrapper(update, context, *args, **kwargs):
        user = update.effective_user
        chat = update.effective_chat

        is_admin = await is_user_admin(chat, user.id)
        is_named_user = (
            str(user.id) in allowed_users or
            (user.username and f"@{user.username.lower()}" in [u.lower() for u in allowed_users])
        )

        if not (is_admin or is_named_user):
            await update.message.reply_text("You need to be an admin or authorized user to use this command.")
            return

        return await func(update, context, *args, **kwargs)
    return wrapper

async def is_user_admin(chat, user_id):
    administrators = await chat.get_administrators()
    return any(admin.user.id == user_id for admin in administrators)

async def fetch_keyword_lists():
    global check_list1, check_list2, check_list3, allowed_users
    try:
        async with aiohttp.ClientSession() as session:
            async with session.get(KEYWORD_LIST_URL) as response:
                if response.status == 200:
                    data = await response.json()
                    check_list1 = data.get("check_list1", [])
                    check_list2 = data.get("check_list2", [])
                    check_list3 = data.get("check_list3", [])
                    allowed_users = data.get("allowed_users", [])
                    print("Keyword lists updated successfully.")
                else:
                    print(f"Failed to fetch keyword lists. HTTP Status: {response.status}")
    except Exception as e:
        print(f"Error fetching keyword lists: {e}")

@admin_only
async def manual_refresh_keywords(update: Update, context: ContextTypes.DEFAULT_TYPE):
    await fetch_keyword_lists()
    await update.message.reply_text("Keyword lists refreshed manually.")

@admin_only
async def start(update, context):
    await update.message.reply_text(
        "Gate Keeper is active \n\n"
        "Be aware there is a 3 strike warning process in place. \n\n"
        "Any user that receives 3 warnings will be automatically removed from the group."
    )

async def welcome_new_user(update, context):
    if update.message.new_chat_members:
        for user in update.message.new_chat_members:
            user_id = user.id
            chat_id = update.message.chat.id
            first_name = user.first_name

            try:
                user_photos = await context.bot.get_user_profile_photos(user_id)
                pic_count = user_photos.total_count
            except:
                pic_count = 0

            if pic_count == 0:
                await context.bot.send_photo(
                    chat_id=chat_id,
                    photo='https://i.imgur.com/q3l7yOM.jpeg',
                    caption=(
                        f"Welcome *{user.username}*\n\n"  # Bold username in the welcome message
                        "Please ensure you have a profile picture as it's a requirement of this group. "
                        "You will be automatically removed from this group if you do not have a profile picture within 24 hours of joining."
                    ),
                    parse_mode='MarkdownV2'  # Use MarkdownV2 for formatting
                )
            else:
                await context.bot.send_message(
                    chat_id=chat_id,
                    text=(
                        f"Welcome *{user.username}*\n\n"  # Bold username in the message
                        "Please see pinned post for installation instructions."
                    ),
                    parse_mode='MarkdownV2'  # Use MarkdownV2 for formatting
                )

            new_users[user_id] = {"chat_id": chat_id, "join_time": datetime.now()}

@admin_only
async def warn_user(update, context):
    if update.message.reply_to_message:
        warned_user = update.message.reply_to_message.from_user
        target_id = warned_user.id
        chat_id = update.message.chat.id

        user_warnings[target_id] = user_warnings.get(target_id, 0) + 1
        warnings = user_warnings[target_id]

        await update.message.reply_text(
            f"*{warned_user.first_name}* has been warned. \nTotal warnings: {warnings}.",  # Bold username in the warning message
            parse_mode='MarkdownV2'  # Use MarkdownV2 for formatting
        )

        if warnings >= 3:
            try:
                await context.bot.ban_chat_member(chat_id, target_id)
                await context.bot.send_message(
                    chat_id, f"*{warned_user.first_name}* has been removed for receiving 3 warnings.",  # Bold username
                    parse_mode='MarkdownV2'  # Use MarkdownV2 for formatting
                )
                del user_warnings[target_id]
            except Exception as e:
                await update.message.reply_text(f"Failed to remove *{warned_user.first_name}*. Error: {e}", parse_mode='MarkdownV2')
    else:
        await update.message.reply_text("Please reply to a user's message to warn them.")

@admin_only
async def reset_warnings(update, context):
    if update.message.reply_to_message:
        warned_user = update.message.reply_to_message.from_user
        target_id = warned_user.id

        if target_id in user_warnings:
            del user_warnings[target_id]
            await update.message.reply_text(f"Warnings for *{warned_user.first_name}* have been reset.", parse_mode='MarkdownV2')
        else:
            await update.message.reply_text(f"*{warned_user.first_name}* has no warnings.", parse_mode='MarkdownV2')
    else:
        await update.message.reply_text("Please reply to a user's message to reset their warnings.")

@admin_only
async def ban_user(update, context):
    if update.message.reply_to_message:
        banned_user = update.message.reply_to_message.from_user
        target_id = banned_user.id
        chat_id = update.message.chat.id

        reason = " ".join(context.args) if context.args else "No reason provided."

        try:
            await context.bot.ban_chat_member(chat_id, target_id)
            await context.bot.send_message(
                chat_id, f"*{banned_user.first_name}* has been banned. \nReason: {reason}",  # Bold username
                parse_mode='MarkdownV2'  # Use MarkdownV2 for formatting
            )
        except Exception as e:
            await update.message.reply_text(f"Failed to ban *{banned_user.first_name}*. Error: {e}", parse_mode='MarkdownV2')
    else:
        await update.message.reply_text("Please reply to a user's message to ban them.")

@admin_only
async def add_profile_pic(update, context):
    first_name = update.effective_user.first_name
    chat_id = update.effective_chat.id

    await context.bot.send_photo(
        chat_id=chat_id,
        photo='https://i.imgur.com/q3l7yOM.jpeg',
        caption=(
            f"*{first_name}*\n\n"  # Bold first name in the caption
            "Please ensure you have a profile picture as it's a requirement of this group. "
            "You will be automatically removed from this group if you do not have a profile picture within 24 hours."
        ),
        parse_mode='MarkdownV2'  # Use MarkdownV2 for formatting
    )

@admin_only
async def manual_scan_warn(update: Update, context: ContextTypes.DEFAULT_TYPE):
    chat = update.effective_chat
    warned = 0

    async for member in chat.get_administrators():  # to avoid self-scan on large groups
        continue

    async for member in chat.get_members():
        user = member.user
        if user.is_bot or user.id in new_users:
            continue
        try:
            photos = await context.bot.get_user_profile_photos(user.id)
            if photos.total_count == 0:
                await context.bot.send_photo(
                    chat.id,
                    photo='https://i.imgur.com/q3l7yOM.jpeg',
                    caption=f"*{user.first_name}*, please set a profile picture within 24 hours to avoid removal.",  # Bold username
                    parse_mode='MarkdownV2'  # Use MarkdownV2 for formatting
                )
                new_users[user.id] = {"chat_id": chat.id, "join_time": datetime.now()}
                warned += 1
        except Exception as e:
            print(f"Failed to check user {user.id}: {e}")
    await update.message.reply_text(f"Warned {warned} users without profile pictures.")

@admin_only
async def manual_scan_ban(update: Update, context: ContextTypes.DEFAULT_TYPE):
    chat = update.effective_chat
    banned = 0

    async for member in chat.get_members():
        user = member.user
        if user.is_bot:
            continue
        try:
            photos = await context.bot.get_user_profile_photos(user.id)
            if photos.total_count == 0:
                await context.bot.ban_chat_member(chat.id, user.id)
                await context.bot.send_message(
                    chat.id, f"*{user.first_name}* was removed for not having a profile picture.",  # Bold username
                    parse_mode='MarkdownV2'  # Use MarkdownV2 for formatting
                )
                banned += 1
        except Exception as e:
            print(f"Failed to ban user {user.id}: {e}")
    await update.message.reply_text(f"Banned {banned} users without profile pictures.")

async def handle_message(update, context):
    if not update.message or not update.message.text:
        return

    text = update.message.text.lower()
    user_id = update.message.from_user.id
    first_name = update.message.from_user.first_name
    chat_id = update.message.chat.id

    async def issue_warning():
        user_warnings[user_id] = user_warnings.get(user_id, 0) + 1
        warnings = user_warnings[user_id]

        await update.message.reply_text(f"Warning *{first_name}*\nThis is warning {warnings}\\.", parse_mode='MarkdownV2')

        if warnings >= 3:
            try:
                await context.bot.ban_chat_member(chat_id, user_id)
                await context.bot.send_message(chat_id, f"*{first_name}* has been removed for receiving 3 warnings.", parse_mode='MarkdownV2')
                del user_warnings[user_id]
            except Exception as e:
                await update.message.reply_text(f"Failed to remove *{first_name}*. Error: {e}", parse_mode='MarkdownV2')

    if any(check in text for check in check_list1):
        await update.message.reply_text(
            f"Warning *{first_name}* \n\n"
            "The answer is always the same:\n\n"
            "If you've looked, you know the answer\\.\\.\n\n"
            "If you've not looked, it's not there\\.\\.\n\n"
            "*If\\-Its\\-On\\-Its\\-In*",
            parse_mode='MarkdownV2'  # Use MarkdownV2 for formatting
        )
        await issue_warning()

    elif any(check in text for check in check_list2):
        await update.message.reply_text(
            f"Warning *{first_name}* \n\n"
            "Replays are added and announced as they become available\\.",
            parse_mode='MarkdownV2'  # Use MarkdownV2 for formatting
        )
        await issue_warning()

    elif any(check in text for check in check_list3):
        await update.message.reply_text(
            f"*{first_name}*\nPlease see pinned post for installation instructions\\.",  # Bold username
            parse_mode='MarkdownV2'  # Use MarkdownV2 for formatting
        )

async def remove_users_without_profile_pics(context: ContextTypes.DEFAULT_TYPE):
    now = datetime.now()
    bot = context.bot
    to_remove = []

    for user_id, data in new_users.items():
        join_time = data['join_time']
        chat_id = data['chat_id']

        if (now - join_time).total_seconds() >= 86400:
            try:
                user_photos = await bot.get_user_profile_photos(user_id)
                if user_photos.total_count == 0:
                    await bot.ban_chat_member(chat_id, user_id)
                    await bot.send_message(
                        chat_id,
                        f"User ID {user_id} has been removed for not setting a profile picture\\.",
                        parse_mode='MarkdownV2'
                    )
                to_remove.append(user_id)
            except Exception as e:
                print(f"Error checking/removing user {user_id}: {e}")

    for uid in to_remove:
        del new_users[uid]

async def run_bot():
    application = Application.builder().token(BOT_TOKEN).build()

    await fetch_keyword_lists()

    application.job_queue.run_repeating(lambda _: asyncio.create_task(fetch_keyword_lists()), interval=7200)
    application.job_queue.run_repeating(remove_users_without_profile_pics, interval=16200)

    application.add_handler(CommandHandler("start", start))
    application.add_handler(CommandHandler("warn", warn_user))
    application.add_handler(CommandHandler("reset", reset_warnings))
    application.add_handler(CommandHandler("ban", ban_user))
    application.add_handler(CommandHandler("profile", add_profile_pic))
    application.add_handler(CommandHandler("refresh", manual_refresh_keywords))
    application.add_handler(CommandHandler("scan", manual_scan_warn))
    application.add_handler(CommandHandler("clean", manual_scan_ban))

    application.add_handler(MessageHandler(filters.StatusUpdate.NEW_CHAT_MEMBERS, welcome_new_user))
    application.add_handler(MessageHandler(filters.TEXT & ~filters.COMMAND, handle_message))

    print("Bot is running...")
    await application.run_polling()

def main():
    nest_asyncio.apply()
    asyncio.get_event_loop().run_until_complete(run_bot())

if __name__ == "__main__":
    main()